#!/usr/bin/env node

/**
 * Module dependencies.
 */
const app          = require('../app'),
      cluster      = require('cluster'),
      debug        = require('debug')('extemplate:server'),
      http         = require('http'),
      numCPUs      = require('os').cpus().length,
      winston      = require('winston'),
      isProduction = (process.env.NODE_ENV
          && process.env.NODE_ENV.trim() === 'production') ? true : false;

let server, numWorkers;

/**
 * Get port from environment and store in Express.
 */
const port = normalizePort(process.env.PORT || 'VPORT');
app.set('port', port);

/**
 * Workers
 */
numWorkers = process.env.WEB_CONCURRENCY;

if (typeof numWorkers !== 'undefined') {
    numWorkers = parseInt(numWorkers, 10);
    if (isNaN(numWorkers)) {
        numWorkers = numCPUs;
    } else {
        if (numWorkers > numCPUs) {
            numWorkers = numCPUs;
        }
    }
} else {
    numWorkers = numCPUs;
}

/**
 * Cluster
 */
if (cluster.isMaster && isProduction && numWorkers !== 0) {
    winston.log('info', 'Available CPUs %d', numCPUs);
    winston.log('info', 'Master %d is running', process.pid);
    winston.log('info', 'Starting %d worker(s)', numWorkers);

    // Fork workers
    //
    for (let i = 0; i < numWorkers; i++) {
        cluster.fork();
    }

    // If a worker dies, log it to the console and start another worker
    //
    cluster.on('exit', function(worker, code, signal) {
        winston.log('warn', 'Worker %d died', worker.process.pid);
        cluster.fork();
    });

    // Log when a worker starts listening
    //
    cluster.on('listening', function(worker, address) {
        winston.log('info', 'Worker %d started', worker.process.pid);
    });
} else {
    /**
     * Create HTTP server.
     */

    server = http.createServer(app);

    /**
     * Listen on provided port, on all network interfaces.
     */

    server.listen(port);
    server.on('error', onError);
    server.on('listening', onListening);
}

/**
 * Normalize a port into a number, string, or false.
 * @param {string} val
 * @return {number|boolean} port or false
 */
function normalizePort(val) {
  let port = parseInt(val, 10);

  if (isNaN(port)) {
    // named pipe
    return val;
  }

  if (port >= 0) {
    // port number
    return port;
  }

  return false;
}

/**
 * Event listener for HTTP server "error" event.
 * @param {object} error
 */
function onError(error) {
  if (error.syscall !== 'listen') {
    throw error;
  }

  let bind = typeof port === 'string'
    ? 'Pipe ' + port
    : 'Port ' + port;

  // handle specific listen errors with friendly messages
  switch (error.code) {
    case 'EACCES':
      console.error(bind + ' requires elevated privileges');
      process.exit(1);
      break;
    case 'EADDRINUSE':
      console.error(bind + ' is already in use');
      process.exit(1);
      break;
    default:
      throw error;
  }
}

/**
 * Event listener for HTTP server "listening" event.
 */
function onListening() {
  let addr = server.address();
  let bind = typeof addr === 'string'
    ? 'pipe ' + addr
    : 'port ' + addr.port;
  debug('Listening on ' + bind);
}
